from coverdl.providers.provider import Provider
from coverdl.providers.itunes import ITunesProvider
from coverdl.providers.source import Source
import requests

class AppleMusicProvider(ITunesProvider):
    def __init__(self):
        super().__init__(Source.APPLE_MUSIC)

    def _transform_url(self, itunes_url, country='us'):
        # Convert the iTunes url into a higher-quality Apple Music equivalent
        cover_url = itunes_url.replace("is1-ssl", "a1")
        cover_url = cover_url.replace("/image/", f"/{country}/")
        cover_url = cover_url.replace("/thumb/", "/r1000/063/")

        cover_url = '/'.join(cover_url.split('/')[:-1])

        return cover_url
    
    def _test_url(self, url):
        r = requests.head(url)

        return r.ok

    def get_covers(self, artist, album, country='us'):
        results = []
        covers = super().get_covers(artist, album, country)

        for cover in covers:
            cover.cover_url = self._transform_url(cover.cover_url, country)
            if self._test_url(cover.cover_url):
                results.append(cover)

        return results


import requests
from coverdl.providers.provider import Provider, Cover
from coverdl.providers.source import Source
from coverdl.exceptions import ProviderRequestFailed

class DeezerProvider(Provider):
    def __init__(self, base_url="https://api.deezer.com", source=Source.DEEZER):
        super().__init__(base_url, source)

    def get_covers(self, artist, album) -> list[Cover]:
        results = []
        params={
            "q": f"artist:\"{artist}\" album:\"{album}\""
        }
        r = requests.get(self.base_url + "/search/album", params=params)

        if r.ok:
            data = r.json()

            for item in data["data"]:
                results.append(
                    Cover(
                        artist=item["artist"]["name"],
                        title=item["title"],
                        source=self.source,
                        cover_url=item["cover_xl"]
                    )
                )
        else:
            raise ProviderRequestFailed(self.source, r.text)
        
        return results


from coverdl.exceptions import ProviderRequestFailed
from coverdl.providers.provider import Provider, Cover
from coverdl.providers.source import Source
from difflib import SequenceMatcher
import requests

class ITunesProvider(Provider):
    def __init__(self, source=Source.ITUNES):
        super().__init__("https://itunes.apple.com", source)

    def get_covers(self, artist, album, country='us'):
        results = []
        params = {
            "term": artist,
            "media": "music",
            "entity": "album"
        }
        r = requests.get(self.base_url + "/search", params=params)

        if r.ok:
            data = r.json()

            for item in data["results"]:
                # Check if the album names are somewhat similar.
                # TODO: Filter out possible identifiers in the name
                similarity_ratio = SequenceMatcher(None, item["collectionName"].lower().strip(), album.lower().strip()).ratio()
                if similarity_ratio > 0.8:
                    results.append(
                        Cover(
                            artist=item["artistName"],
                            title=item["collectionName"],
                            source=self.source,
                            cover_url=item.get("artworkUrl100") or item.get("artworkUrl60"),
                            confidence=similarity_ratio
                        )
                    )

            return sorted(results, key=lambda c: c.confidence, reverse=True)
        else:
            raise ProviderRequestFailed(self.source, r.text)


from PIL import Image
import requests
import os
import mimetypes
import imagehash
from coverdl.metadata import SUPPORTED_SONG_EXTENSIONS

IMAGE_EXTENSIONS = [".jpg", ".jpeg", ".png"]

def download_cover(url, target, cover_name):
    mimetype = mimetypes.guess_type(url)[0]
    ext = mimetypes.guess_extension(mimetype)

    r = requests.get(url, timeout=10)
    r.raise_for_status()

    with open(os.path.join(target, cover_name + ext), 'wb') as f:
        f.write(r.content)

def compare_covers(img1, img2):
    hash_0 = imagehash.average_hash(Image.open(img1))
    hash_1 = imagehash.average_hash(Image.open(img2))

    return hash_0 - hash_1

def get_paths_with_covers(path):
    paths = []
    for root, dirs, _ in os.walk(path):
        for current_dir in dirs:
            full_dir = os.path.join(root, current_dir)
            if is_album_dir(full_dir):
                paths.append(full_dir)
    return paths

def get_recursive_paths(path):
    paths = []
    for artist_path in os.listdir(path):
        for album_path in os.listdir(os.path.join(path, artist_path)):
            album_path = os.path.join(path, artist_path, album_path)
            if os.path.isdir(album_path):
                paths.append(album_path)
    return paths

def get_base_path(path):
    if os.path.isfile(path):
        return os.path.dirname(path)
    return os.path.normpath(path)

def get_cover(path):
    if os.path.isfile(path):
        path = os.path.dirname(os.path.abspath(path))

    files = os.listdir(path)

    for file in files:
        name, ext = os.path.splitext(file)
        if ext in IMAGE_EXTENSIONS and name in ["folder", "poster", "cover", "default"]:
            return os.path.join(path, file)

def has_song(dir):
    for f in os.listdir(dir):
        _, ext = os.path.splitext(f)
        if ext in SUPPORTED_SONG_EXTENSIONS:
            return True
    return False

def has_cover(path):
    return bool(get_cover(path))

def is_album_dir(dir):
    return has_cover(dir) and has_song(dir)


from abc import ABC, abstractmethod
from coverdl.providers.source import Source
from dataclasses import dataclass

@dataclass
class Cover:
    artist: str
    title: str
    source: Source
    cover_url: str
    confidence: int = 1

class Provider(ABC):
    base_url: str
    source: Source

    def __init__(self, base_url: str, source: Source):
        self.base_url = base_url
        self.source = source

    @abstractmethod
    def get_covers(self, artist: str, album: str, country: str) -> list[Cover]:
        pass


from .source import Source
from .provider import Provider
from .deezer import DeezerProvider
from .itunes import ITunesProvider
from .apple_music import AppleMusicProvider

providers: list[Provider] = [
    DeezerProvider(),
    ITunesProvider(),
    AppleMusicProvider()
]